<?php
/*
 * User-friendly interface to SIEVE server-side mail filtering.
 * Plugin for Squirrelmail 1.2+
 *
 * Copyright (c) 2002 Alexandros Vellis <avel@users.sourceforge.net>
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 */

/*	makesinglerule: Get a $sess[] array and build _part of_
 *			a SIEVE rule.
 *			$rule = a rule array
 *			$type = text|rule
 *			$default = 0 or 1
 *
 *	makesieverule:	Get an array of sess['blah'] arrays and build
 *			our rule.
 *
 *	It's all kludgy.
 */

function makesinglerule($rule, $type="rule") {

/* Arguments are as follows:

	$rule	just a rule array.
	
	$type	verbose = return a (verbose) textual description of the rule.
		terse = return a very terse description
		rule = return a string with the appropriate SIEVE code.
	
	$default boolean - is this a default rule ("else") or a normal rule
		 (if.... elseif...)
*/

global $maxitems;

/* Step zero :-) : serialize & encode my array */

$coded = urlencode(base64_encode(serialize($rule)));

$out = "#START_SIEVE_RULE".$coded."END_SIEVE_RULE\n";

/* Step one: make the if clause */

/* The actual 'if' will be added by makesieverule() */

$terse = '<table width="100%" border="0" cellspacing="2" cellpadding="2"><tr><td align="left">';

if($rule['type']=="4") {
 	$text = _("For <strong>ALL</strong> incoming messages; ");
	$terse .= "ALL";
} else {
	$text = "<strong>"._("If")."</strong> ";
} 

switch ($rule['type']) {
case "1":	/* address --- deprecated */
	for ( $i=0; $i<3; $i++) {
		$out .= 'address :'.${'address'.$i};
		if(${'addressrel'.$i} != "0") {
			$out .= ":";
		}
	}
	break;

case "2":	/* header */
	if(isset($rule['condition'])) {
	switch ($rule['condition']) {
		case "or":
			$out .= "anyof (";
			$text .= _("<em>any</em> of the following mail headers match: ");
			// $terse .= "ANY (";
			break;
		case "and":
			$out .= "allof (";
			$text .= _("<em>all</em> of the following mail headers match: ");
			// $terse .= "ALL (";
			break;
		default: /* condition was not defined, so there's only one header item. */
			$lonely = true;
			break;
	}
	}
	/* if ( $i<sizeof($rule['headermatch'][$i] > 1) {
		$out .=
	}
	*/
	for ( $i=0; $i<sizeof($rule['headermatch']); $i++) {
		$text .= _("the header");
		$text .= " &quot;".$rule['header'][$i]."&quot; ";
		$terse .= " ".$rule['header'][$i]." ";
		$escapeslashes = false;

 		switch ($rule['matchtype'][$i]) {
 			case "is":
 				$out .= "header :is";
				$text .= _("is");
				$terse .= "=";
 				break 1;
 			case "is not":
 				$out .= "not header :is";
				$text .= _("is not");
				$terse .= "!=";
 				break 1;
 			case "contains":
 				$out .= "header :contains";
				$text .= _("contains");
				$terse .= "=";
 				break 1;
 			case "does not contain":
 				$out .= "not header :contains";
				$text .= _("does not contain");
				$terse .= "!~=";
 				break 1;
 			case "matches":
 				$out .= "header :matches";
				$text .= _("matches");
				$terse .= "M=";
				$escapeslashes = true;
 				break 1;
 			case "does not match":
 				$out .= "not header :matches";
				$text .= _("does not match");
				$terse .= "!M=";
				$escapeslashes = true;
 				break 1;
 			case "gt":
				$out .= "header :value \"gt\" :comparator \"i;ascii-numeric\"";
				$text .= _("is greater than");
				$terse .= ">";
 				break 1;
 			case "ge":
				$out .= "header :value \"ge\" :comparator \"i;ascii-numeric\"";
				$text .= _("is greater or equal to");
				$terse .= ">=";
 				break 1;
 			case "lt":
				$out .= "header :value \"lt\" :comparator \"i;ascii-numeric\"";
				$text .= _("is lower than");
				$terse .= "<";
 				break 1;
 			case "le":
				$out .= "header :value \"le\" :comparator \"i;ascii-numeric\"";
				$text .= _("is lower or equal to");
				$terse .= "<=";
 				break 1;
 			case "eq":
				$out .= "header :value \"eq\" :comparator \"i;ascii-numeric\"";
				$text .= _("is equal to");
				$terse .= "==";
 				break 1;
 			case "ne":
				$out .= "header :value \"ne\" :comparator \"i;ascii-numeric\"";
				$text .= _("is not equal to");
				$terse .= "!=";
 				break 1;
 			case "regex":
 				$out .= "header :regex :comparator \"i;ascii-casemap\"";
				$text .= _("matches the regural expression");
				$terse .= "R=";
				$escapeslashes = true;
 				break 1;
 			case "not regex":
 				$out .= "not header :matches";
				$text .= _("does not match the regural expression");
				$terse .= "!R=";
				$escapeslashes = true;
 				break 1;
 			case "exists":
 				$out .= "exists";
				$text .= _("exists");
				$terse .= "E";
 				break 1;
 			case "not exists":
 				$out .= "not exists";
				$text .= _("does not exist");
				$terse .= "!E";
 				break 1;
 			default:
 				break 1;
 		}
		$out .= " \"".$rule['header'][$i]."\" ";

		if ($escapeslashes) {
			$out .= "\"".str_replace('\\', '\\\\\\\\', $rule['headermatch'][$i])."\"";
		} else {
			$out .= "\"".$rule['headermatch'][$i]."\"";
		}

		$text .= " &quot;".htmlspecialchars($rule['headermatch'][$i])."&quot;";

 		if ($rule['matchtype'][$i] == "contains") {
			$terse .= " *".htmlspecialchars($rule['headermatch'][$i])."* ";
		} else {
			$terse .= " ".htmlspecialchars($rule['headermatch'][$i])." ";
		}

		if(isset($rule['headermatch'][$i+1])) {
			$out .= ",\n";
			$text .= ", ";

			if ($rule['condition'] == "or" ) {
				$terse .= " OR<br />";
			} elseif ($rule['condition'] == "and" ) {
				$terse .= " AND<br />";
			}
		} elseif($i == 0  && !isset($rule['headermatch'][1]) ) {
		// && ($lonely == true)
			$out .= "\n";
			$text .= ", ";
		} else {
			$out .= ")\n";
			$text .= ", ";
		}

	}
	
	/* if ($rule['headerrel0'] == "0") {
		$text .= " \"".$rule['header0']."\" ".$rule['matchtype0']." \"".$rule['headermatch0']."\", ";
	} else {
		for ($i=0; $i<sizeof($rule['headermatch']); $i++) {
			$text .= _("the header");
			$text .=' &quot;'.$rule['header'][$i].'" '.$rule['matchtype'][$i].' "'.$rule['headermatch'][$i].'&quot;, ';
			if ($rule['headerrel'][$i]=="0") {
				break;
			} 
		}		
	} */
	break;

case "3":	/* size */
	$out .= 'size :';
	$text .= _("the size of the message is");
	$text .= "<em>";
	$terse .= "SIZE";
	
	if($rule['sizerel'] == "bigger") {
		$out .= "over ";
		$terse .= " > ";
		$text .= _(" bigger");
	} else {
		$out .= "under ";
		$terse .= " < ";
		$text .= _(" smaller");
	}
	$text .= " "._("than")." ".$rule['sizeamount']." ".$rule['sizeunit']."</em>, ";
	
	$terse .= $rule['sizeamount'];
	$out .= $rule['sizeamount'];
	
	if($rule['sizeunit']=="kb") {
		$out .= "K\n";
		$terse .= "K\n";
	} elseif($rule['sizeunit']=="mb") {
		$out .= "M\n";
		$terse .= "M\n";
	}
	break;

case "4":	/* always */
	$out .= "true\n";
	break;
}



/* step two: make the then clause */

$out .= "{\n";
$terse .= '</td><td align="right">';

if($rule['type']!="4") {
	$text .= "<strong>";
	$text .= _("then");
	$text .= "</strong> ";
}

switch ($rule['action']) {
case "1":	/* keep (default) */
	$out .= "keep;";
	$text .= _("<em>keep</em> it.");
	$terse .= "KEEP";
	break;

case "2":	/* discard */
	$out .= "discard;";
	$text .= _("<em>discard</em> it.");
	$terse .= "DISCARD";
	break;

case "3":	/* reject w/ excuse */
	
	$out .= "reject text:\n".$rule['excuse']."\r\n.\r\n;";
	$text .= _("<em>reject</em> it, sending this excuse back to the sender:")." \"".$rule['excuse']."\".";
	$terse .= "REJECT";
	break;

case "4":	/* redirect to address */
	$out .= "redirect \"".$rule['redirectemail']."\";";
	$text .= _("<em>redirect</em> it to the email address")." ".$rule['redirectemail'].".";
	$terse .= "REDIRECT ".$rule['redirectemail'];
	break;

case "5":	/* fileinto folder */

	$out .= 'fileinto "'.$rule['folder'].'";';
	$text .= _("<em>file</em> it into the folder <strong>");
	$text .= imap_utf7_decode_local($rule['folder']);
	$text .= "</strong>.";
	$terse .= "FILEINTO ".imap_utf7_decode_local($rule['folder']);
	break;
case "6":      /* vacation message */
	/* Check if $addresses is valid */
	if( !isset($rule['vac_addresses']) || trim($rule['vac_addresses'])=="" ) {
		global $data_dir, $username;
		$addresses = getPref($data_dir, $username, 'email_address');
	} else {
		$addresses = str_replace(",",'","',str_replace(" ","",$rule['vac_addresses']));
	}
 	$out .= 'vacation :days '.$rule['vac_days'].' :addresses ["'.$addresses.
	'"] '." text:\n".$rule['vac_message']."\r\n.\r\n;";

 	/* Used to be: '"] "'.$rule['vac_message'].'";'; */
 	$text .= _("reply with this vacation message: ").$rule['vac_message'];
	$terse .= "VACATION";
 	break;
default:
	return false;
	break;
}

if (isset($rule['keepdeleted'])) {
	$text .= _(" Also keep a copy in INBOX, marked as deleted.");
	$out .= "\naddflag \"\\\\\\\\\\\\\\\\Deleted\";\nkeep;";
	$terse .= "<br />KEEP DELETED";
}

if (isset($rule['stop'])) {
	$text .= _(" Then <strong>STOP</strong> processing rules.");
	$out .= "\nstop;";
	$terse .= "<br />STOP";
}

/* Notify extension:

'method' => string
'id' => string
'options' => array( [0]=> foo, [1] => bar )
'priority' => low|normal|high
'message' => string


notify :method "mailto"
:options "koula@intra.com"
:message "Sou hrthe ena mail apo .... lala" ;

*/

if (array_key_exists("notify", $rule) && is_array($rule['notify']) && ($rule['notify']['method'] != '')) {
	global $notifystrings, $prioritystrings;
	$text .= _(" Also notify using the method")
		. " <em>" . $notifystrings[$rule['notify']['method']] . "</em>, ".
		_("with")
		. " " . $rule['notify']['priority'] . " " .
		_("priority and the message")
		. " <em>&quot;" . $rule['notify']['message'] . "&quot;</em>.";
		
	$out .= "\nnotify :method \"".$rule['notify']['method']."\" ";
	
	$out .= ":options \"".$rule['notify']['options']."\" ";

	if(isset($rule['notify']['id'])) {
		$out .= ":id \"".$rule['notify']['id']."\" ";
	}
	if(isset($rule['notify']['priority']) && array_key_exists($rule['notify']['priority'], $prioritystrings)) {
		$out .= ":".$rule['notify']['priority'] . " ";
	}
	$out .= ":message \"".$rule['notify']['message']."\"";
	$out .= ";\n";

	$terse .= "<br />NOTIFY";
}

$out .= "\n}";
$terse .= "</td></tr></table>";

if ($type == "terse") {
	return $terse;
} elseif (($type == "text") || ($type == "verbose")) {
	return $text;
} else {
	return $out;
}

}


/* -------------------------------------------------- */


/* Make a complete set of rules, i.e. a SIEVE script */
/* $rulearray is an array of associative arrays describing a rule each. */
/* $defaultrulearray is an array that should have only one element,
 * $defaultrulearray[0], of the default rule */

function makesieverule ($rulearray) {
	global $implemented_capabilities, $cap_dependencies;

	if ( (sizeof($rulearray) == 0) || $rulearray[0] == "0" ) {
		return false;
	}

	foreach($implemented_capabilities as $no=>$cap) {
		if(in_array($cap, $_SESSION['sieve_capabilities'])) {
			$torequire[] = $cap;
			if(array_key_exists($cap, $cap_dependencies)) {
				foreach($cap_dependencies[$cap] as $no2=>$dep) {
					$torequire[] = $dep;
				}
			}
		}
	}
		
 	$out = "require [";
	for($i=0; $i<sizeof($torequire); $i++) {
		$out .= '"'.$torequire[$i].'"';
		if($i != (sizeof($torequire) -1) ) {
			$out .= ',';
		}
	}
	$out .= "];\n";

	for ($i=0; $i<sizeof($rulearray); $i++) {
		switch ($i) {
			case 0:				$out .= "if\n";		break;
			/* case sizeof($rulearray): 	$out .= "\nelse\n";	break; */
 			default:			$out .= "\nif\n";	break;
		}		
		$out .= makesinglerule($rulearray[$i],"rule");
	}
	
	/*
	print "The script is: <pre>";
	print $out;
	print "</pre>"; 
	*/
	return $out;
}

?>
