<?php
/*
 * User-friendly interface to SIEVE server-side mail filtering.
 * Plugin for Squirrelmail 1.2+
 *
 * Copyright (c) 2002 Alexandros Vellis <avel@users.sourceforge.net>
 *
 * Based on Dan Ellis' test scripts that came with sieve-php.lib
 * <danellis@rushmore.com> <URL:http://sieve-php.sourceforge.net>
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 */

/* table.php: main routine that shows a table of all the rules and allows
 * manipulation. */

define('SM_PATH','../../');
require_once(SM_PATH . 'include/validate.php');
require_once(SM_PATH . 'include/load_prefs.php');
require_once(SM_PATH . 'functions/page_header.php');
require_once(SM_PATH . 'functions/imap.php');

include "config.php";
require_once "avelsieve_support.inc.php";
require_once "sieve-php.lib.php";
require_once "table_html.php";
require_once "getrule.php";
require_once "buildrule.php";
require_once "sieve.php";

sqsession_is_active();

sqgetGlobalVar('key', $key, SQ_COOKIE);
sqgetGlobalVar('onetimepad', $onetimepad, SQ_SESSION);

if(isset($_SESSION['part'])) {
	session_unregister('part');
}

if(isset($_SESSION['sess'])) {
	session_unregister('sess');
}

$location = get_location();

/* Need the cleartext password to login to timsieved */
$acctpass = OneTimePadDecrypt($key, $onetimepad);

sqgetGlobalVar('rules', $rules, SQ_SESSION);
sqgetGlobalVar('logout', $logout, SQ_POST);

/* $sieve=new sieve($server, $port, $acctname, $acctpass, $adminacct); */
$imap_server =  sqimap_get_user_server ($imapServerAddress, $username);

$sieve=new sieve($imap_server, $sieveport, $username, $acctpass, $username);
//print "DEBUG: Connecting... ($imap_server, $sieveport, $username, $acctpass, $username)";

sqgetGlobalVar('sieve_capabilities', $sieve_capabilities, SQ_SESSION);

if (!isset($rules)) {
	/* If the rules are cached, don't login! */ 

	if ($sieve->sieve_login()){	/* User has logged on */
		// print "Welcome! Login successful.\n";
		if(!isset($sieve_capabilities)) {
			$sieve_capabilities = $sieve->sieve_get_capability();
			// sqsession_register($sieve_capabilities, 'sieve_capabilities');
			 $_SESSION['sieve_capabilities'] = $sieve_capabilities;
		}
	} else {
		$errormsg = _("Could not log on to timsieved daemon on your IMAP server");
		$errormsg .= " " . $imapServerAddress.".<br />";
		$errormsg .= _("Please contact your administrator.");
		print_errormsg($errormsg);
		exit;
	}
}

/* Here we continue normally. */
/* Get phpscript script. If it exists. */

if (!isset($rules)) {

	if($sieve->sieve_listscripts() /* && isset($currentrules) */ ){
		if(!isset($sieve->response)) {
			/* There is no SIEVE script on the server. */
			displayPageHeader($color, 'None');
			printheader2( _("Current Mail Filtering Rules") );
			print_all_sections_start();
			print_section_start(_("No Filtering Rules Defined Yet"));
			print_create_new();
			print_section_end(); 
			print_all_sections_end();
			printfooter();
			printfooter2();
			exit;
			
		} elseif(is_array($sieve->response)){
			$i = "1";
			foreach($sieve->response as $line){
				$scriptname[$i] = $line;
				$i++;
			}
		} else {
			print_errormsg("DEBUG: listscripts returns a single thing? Strange. It doesnt do that usually.");
			if($response == "phpscript") {
				print_phpscript();
				if($sieve->sieve_getscript("phpscript")){
					if(is_array($sieve->response))
						foreach($sieve->response as $line){
							$sieverule .= $line;
						}
	                    		} else {
						print $sieve->response;
					}
	           	 
			} else {
				print_errormsg("ERROR: Obviously you can write SIEVE scripts without this interface! :-)");
				exit;
			}
		
		}
	}
}
/* If we got here, everything should be ok.*/

/* On to the code that executes if phpscript exists or if a new rule has been
 * created. */

if($logout) {
	/* Activate phpscript and log out. */
	
	if (!($sieve->sieve_login())){
		$errormsg = _("Could not log on to timsieved daemon on your IMAP server");
		$errormsg .= " " . $imapServerAddress.".<br />";
		$errormsg .= _("in order to save your rules.") . "<br />";
		$errormsg .= _("Please contact your administrator.");
		print_errormsg($errormsg);
		exit;
	}
	
	require_once("constants.php");

	if ($newscript = makesieverule($rules)) {

		avelsieve_upload_script($newscript);

		if(!($sieve->sieve_setactivescript("phpscript"))){
			/* Just to be safe. */
			$errormsg = _("Could not set active script on your IMAP server");
			$errormsg .= " " . $imapServerAddress.".<br />";
			$errormsg .= _("Please contact your administrator.");
			print_errormsg($errormsg);
			exit;
		}
		$sieve->sieve_logout();
	
	} else {
		/* upload a null thingie!!! :-) This works for now... some time
		 * it will get better. */
		avelsieve_upload_script(""); 
		/* if(sizeof($rules) == "0") {
			avelsieve_delete_script();
		} */
	}
	session_unregister('rules');
	header("Location: ../../src/options.php?optpage=avelsieve\n\n");
	exit;

} elseif (isset($_POST['add'])) {
	/* code to start wizard to add a new script */
	header("Location: $location/addrule.php");
	exit;
}

/* I want to remove this */
/* if(!$returnnewrule) {
	$returnnewrule = "no";
} */

/* Actually GET the script 'phpscript'. */

if (!isset($rules)) {
	$sieverule = '';
	if($sieve->sieve_getscript("phpscript")){
		if(is_array($sieve->response)) {
			foreach($sieve->response as $line){
				$sieverule .= "$line\n";
			}
		} else {
			$prev = bindtextdomain ('avelsieve', SM_PATH . 'plugins/avelsieve/locale');
			textdomain ('avelsieve');
			$errormsg = _("Could not get SIEVE script from your IMAP server");
			$errormsg .= " " . $imapServerAddress.".<br />";
			$errormsg .= _("(Probably the script is size null).");
			$errormsg .= _("Please contact your administrator.");
			print_errormsg($errormsg);
			exit;
		}
	}
	
	/* $sieverule has a SIEVE script. Parse that. */
	$rules = getruledata($sieverule);
	
	if ($rules == "0" && !isset($_SESSION['returnnewrule']) ) {
		/* NO SCRIPT, WHAT HAPPENS IF THERE IS NO SCRIPT. (Stop shouting). */

		displayPageHeader($color, 'None');
		$prev = bindtextdomain ('avelsieve', SM_PATH . 'plugins/avelsieve/locale');
		textdomain ('avelsieve');
		printheader2(false);
		print_all_sections_start();
		print_section_start(_("No Filtering Rules Defined Yet"));
		print_create_new();
		print_section_end(); 
		print_all_sections_end();
		print_buttons();
		printfooter2();
		exit;
	} else {
		$defaultrule = getruledata($sieverule);
		session_register('rules');
		/* print "DEBUG: found the foolowing default rule: <pre>"; print_r($defaultrule); print "</pre>"; */
	}
	/* now, $_SESSION['rules'] is an array of associative arrays of rules. :-) */

}

/* Routine for Delete / Delete selected / edit / duplicate / moveup/down */
if(isset($_GET['rule']) || isset($_POST['deleteselected'])) {
	if (isset($_GET['edit'])) {
		header("Location: $location/edit.php?edit=".$_POST['rule']."");
		exit;

	} elseif (isset($_GET['dup'])) {
		header("Location: $location/edit.php?edit=".$_POST['rule']."&dup=1");
		exit;

	} elseif (isset($_GET['rm']) || ( isset($_POST['deleteselected']) && isset($_POST['selectedrules'])) ) {
		if (isset($_POST['deleteselected'])) {
			$rules2 = $rules;
			foreach($_POST['selectedrules'] as $no=>$sel) {
				unset($rules2[$sel]);
				// $rules = array_del($rules, $sel); 
			} 
			$rules = array_values($rules2);
			$_SESSION['comm']['deleted'] = $_POST['selectedrules'];

		} elseif(isset($_GET['rm'])) {
			$rules = array_del($rules, $_GET['rule']);
			$_SESSION['comm']['deleted'] = $_GET['rule'];
		}


		if(sizeof($_SESSION['rules']) == "0") {
			// print "DEBUG: Ok, size of rules is 0 apparently.";
	
			if (!$conservative) {
				avelsieve_delete_script();
			}
			session_unregister('rules');
			displayPageHeader($color, 'None');
			$prev = bindtextdomain ('avelsieve', SM_PATH . 'plugins/avelsieve/locale');
			textdomain ('avelsieve');
			printheader2(false);
			print_all_sections_start();
			print_section_start(_("All your rules have been deleted"));
			print_create_new();
			print_section_end(); 
			print_all_sections_end();
			print_buttons();
			print_footer();
			printfooter2();
			exit;
		} 


	} elseif (isset($_GET['mvup'])) {
		$rules = array_swapval($rules, $_GET['rule'], $_GET['rule']-1);

	} elseif (isset($_GET['mvdn'])) {
		$rules = array_swapval($rules, $_GET['rule'], $_GET['rule']+1);
	}

	session_register('rules');
	
	/* Register changes to timsieved if we are not conservative in our
	 * connections with him. */

	if (!($conservative && $rules)) {
		$newscript = makesieverule($rules);
		avelsieve_upload_script($newscript);
	}
}	

if (isset($_SESSION['returnnewrule'])) { /* Get the new rule and put it in the script */
	
	$newrule = $_SESSION['returnnewrule'];
	// unserialize(base64_decode(urldecode($returnnewrule)));
	session_unregister('returnnewrule');

	//print "DEBUG: Adding new: ";	print_r($newrule);
	
	if (!is_array($rules)) {
		unset($rules);
		$rules[0] = $newrule;
	} else {
		$rules[] = $newrule;
	}
	
	if ($conservative == false) {
		$newscript = makesieverule($rules);
		avelsieve_upload_script($newscript);
	} 
}

$_SESSION['rules'] = $rules;

// $compose_new_win = getPref($data_dir, $username, 'compose_new_win');

$sieve->sieve_logout();

/* --------------------------------- main --------------------------------- */

/* Printing, part zero: Headers et al */
displayPageHeader($color, 'None');

$prev = bindtextdomain ('avelsieve', SM_PATH . 'plugins/avelsieve/locale');
textdomain ('avelsieve');

require "constants.php";

//print "<pre>SESSION: "; print_r($_SESSION); print "</pre>";
//print "<pre>POST: "; print_r($_POST); print "</pre>";
//print "<pre>COOKIE: "; print_r($_COOKIE); print "</pre>";

if(isset($_GET['mode'])) {
	if(array_key_exists($_GET['mode'], $displaymodes)) {
		$mode = $_GET['mode'];
	} else {
		$mode = "verbose";
	}
	sqsession_register($mode, 'mode');
} else {
	if(isset($_SESSION['mode'])) {
		if(array_key_exists($_SESSION['mode'], $displaymodes)) {
			$mode = $_SESSION['mode'];
		} else {
			$mode = "verbose";
		}
	} else {
		$mode = "verbose";
	}
}

// print_my_header();
printheader2( _("Current Mail Filtering Rules") );
print_all_sections_start();

/*
print "DEBUG:<pre>";
print_r($_SESSION);
print_r($_POST);
print "</pre>";
*/


/* Printing, part one: the table with the rules. */

print '<form name="rulestable" method="POST" action="table.php">';

print_table_header();

$toggle = false;

for ($i=0; $i<sizeof($rules); $i++) {
	print "<tr";
	if ($toggle) {
		print ' bgcolor="'.$color[12].'"';
	}
	print "><td>".($i+1)."</td><td>";
	print '<input type="checkbox" name="selectedrules[]" value="'.$i.'" /></td><td>';
	print makesinglerule($rules[$i], $mode);
	print '</td><td style="white-space: nowrap"><p>';

	/* print '</td><td><input type="checkbox" name="rm'.$i.'" value="1" /></td></tr>'; */
	/* Edit */
	print '<a href="edit.php?rule='.$i.'&amp;edit='.$i.'">';
	if($useimages) {
		print '<img title="'._("Edit").'" src="'.$location.'/images/'.$imagetheme.'/edit.png" alt="'._("Edit").'" border="0" />';
	} else {
		print _("Edit");
		print " | ";
	}
	print '</a>';

	/* Delete */
	print ' <a href="table.php?rule='.$i.'&amp;rm=">';
	if($useimages) {
		print '<img title="'._("Delete").'" src="'.$location.'/images/'.$imagetheme.'/del.png" alt="'._("Delete").'" value="Delete" border="0" />';
	} else {
		print _("Delete");
		print " | ";
	}
	print '</a>';

	/* Duplicate */
	print ' <a href="edit.php?rule='.$i.'&amp;edit='.$i.'&amp;dup=1">';
	if($useimages) {
		print '<img title="'._("Duplicate").'" src="'.$location.'/images/'.$imagetheme.'/dup.png" alt="'._("Duplicate").'" value="Duplicate" border="0" /></a>';
	} else {
		print _("Duplicate");
	}
	print '</a>';

	/* Move up */
	if ($i != 0) {
		print ' <a href="table.php?rule='.$i.'&amp;mvup='.$i.'">';
		if($useimages) {
			print '<img title="'._("Move Up").'" src="'.$location.'/images/'.$imagetheme.'/up.png" alt="'._("Move Up").'" value="Move Up" border="0" />';
		} else {
			print " | ";
			print _("Move Up");
		}
		print '</a>';

	}

	/* Move down */
	if ($i != sizeof($rules)-1 ) {
		print ' <a href="table.php?rule='.$i.'&amp;mvdn='.$i.'">';
		if($useimages) {
			print '<img title="'._("Move Down").'" src="'.$location.'/images/'.$imagetheme.'/down.png" alt="'._("Move Down").'" value="Move Down" border="0" />';
		} else {
			print " | ";
			print _("Move Down");
		}
		print '</a>';
	}

	print '</p></td></tr>';
	if(!$toggle) {
		$toggle = true;
	} elseif($toggle) {
		$toggle = false;
	}
}

print '<tr><td colspan="4">';
	print '<table width="100%" border="0"><tr><td align="left">';
	print '<input type="submit" name="deleteselected" value="' . _("Delete Selected") . '" /> '; 
	print '</td><td align="right">';
	print_addnewrulebutton();
	print '</td></tr></table>'; 
print '</td></tr>';

print_table_footer();
print '</form>';
// print_buttons();

print_all_sections_end();
print_footer();

printfooter2();
//textdomain('squirrelmail');

?>
